@@ -10,6 +10,10 @@
 from itertools import count
 from decimal import Decimal
 from datetime import date, time
+import urllib2
+from contextlib import closing
+from urllib import urlencode
+from functools import partial
 
 try:
     from babel.dates import format_time, parse_time, format_date, parse_date
@@ -25,7 +29,7 @@ from formular._utils import (_get_cls_args, _value_matches_choice,
                              _iter_choices, _to_list, _make_name)
 from formular.widgets import (TextInput, PasswordInput, Checkbox, Textarea,
                               RadioButtonGroup, CheckboxGroup, MappingWidget,
-                              MultipleWidget)
+                              MultipleWidget, ReCAPTCHAWidget, SubmitInput)
 
 def _(message):
     """
@@ -37,6 +41,11 @@ def _(message):
 class FieldMeta(type):
     _dict_names = ["validator_factories", "error_messages"]
 
+    def __new__(cls, name, bases, attributes):
+        for dict_name in cls._dict_names:
+            attributes[dict_name] = attributes.get(dict_name, {})
+        return type.__new__(cls, name, bases, attributes)
+
     def __call__(self, *args, **kwargs):
         dicts = dict((dict_name, {}) for dict_name in self._dict_names)
         for base in self.mro():
@@ -73,9 +82,9 @@ class Field(object):
     passed as key with the initial value as value.
 
     Available validators for this field are:
-        - :func:`formular.validators.required`
-        - :func:`formular.validators.requires`
-        - :func:`formular.validators.equals`
+        - :func:`~formular.validators.required`
+        - :func:`~formular.validators.requires`
+        - :func:`~formular.validators.equals`
 
     Validators and error messages are inherited by subclasses.
     """
@@ -86,17 +95,17 @@ class Field(object):
     #: validator factories as returned by
     #: :func:`formular.validators.as_validator`.
     validator_factories = {
-        "required": validators.required,
-        "requires": validators.requires,
-        "equals": validators.equals
+        "required" : validators.required,
+        "requires" : validators.requires,
+        "equals"   : validators.equals
     }
 
     #: Maps the validator names to the corresponding error messages.
     error_messages = {
-        "invalid": _(u"The value is invalid."),
-        "required": _(u"This field is required."),
-        "requires": _(u"Requires {{form.fields[{initial_value}].label}}."),
-        "equals": _(u"Does not equal {{form.fields[{initial_value}].label}}.")
+        "invalid"  : _(u"The value is invalid."),
+        "required" : _(u"This field is required."),
+        "requires" : _(u"Requires {{form.fields[{initial_value}].label}}."),
+        "equals"   : _(u"Does not equal {{form.fields[{initial_value}].label}}.")
     }
 
     #: The default value for this field type. Default values may be mutable so
@@ -107,12 +116,17 @@ class Field(object):
     widget = TextInput
 
     @classmethod
-    def register_as_validator(cls, obj):
+    def register_as_validator(cls, *args, **kwargs):
         """
         Decorator to add a validator.
 
-        :param obj: Either a name or a callable object with a ``__name__``
-                    attribute.
+        :param name_or_callable:
+            Either a name or a callable object with a ``__name__`` attribute.
+
+        :param message:
+            The error message for this validator.
+
+            .. versionadded:: 0.2
 
         This method can be used like this::
 
@@ -120,23 +134,49 @@ class Field(object):
             def required(form, is_required, value):
                 return bool(value) if is_required else True
 
-        However you might not wanna have a `required` - or whatever the name is
-        - function lying around somewhere, so you can just
-        define the name explicitly::
+        Or if you want to pass the message directly::
+
+            @Field.register_as_validator(message=u"This field is required")
+            ...
+
+        However you might not want to have a `required` - or whatever the name
+        is - function lying around somewhere, so you can just define the name
+        explicitly::
 
             @Field.register_as_validator("required")
             def a_name_i_like_a_lot_more(form, is_required, value):
                 return bool(value) if is_required else True
 
+        Of course you can also pass the `message` parameter here.
+
         .. note:: If you register a validator on a field class, this validator
-                  will be registered on every subclass of this field.
+                  will be registered on every subclass of this field. The same
+                  goes for error messages
         """
-        def decorate(func, name=obj):
+        message = kwargs.pop("message", None)
+        name = None
+        func = None
+        if kwargs:
+            raise TypeError(
+                "unexpected keyword argument: {0}".format(kwargs.popitem()[0])
+            )
+        if args:
+            if isinstance(args[0], basestring):
+                name = args[0]
+            else:
+                func = args[0]
+            if args[1:]:
+                raise TypeError("expected at most one positional argument")
+        def decorate(func, name, message):
+            if name is None:
+                name = func.__name__
+            if message:
+                cls.error_messages[name] = message
             cls.validator_factories[name] = validators.as_validator(func)
             return func
-        if not isinstance(obj, basestring):
-            return decorate(obj, name=obj.__name__)
-        return decorate
+        if func is None:
+            return partial(decorate, name=name, message=message)
+        return decorate(func, name, message)
 
     def __init__(self, name=None, label=None, default=missing,
                  error_messages=None, **validators):
@@ -264,10 +304,19 @@ class Field(object):
             error_message = error_message.format(**validator_args)
         return error_message
 
-    def validate(self, value):
+    def validate(self, value, convert=True):
         """
-        Parses and validates `value`. If the validation fails with one or more
-        errors this method returns ``False`` otherwise ``True``.
+        Converts and validates `value`. If the validation fails with one or
+        more errors this method returns ``False`` otherwise ``True``.
+
+        :param value:
+            A primitive or converted value.
+
+        :param convert:
+            If the given `value` is already converted this has to be set to
+            ``False``.
+
+            .. versionadded:: 0.2
 
         The parsed value can be accessed through the `value` attribute, the
         errors are stored as a :class:`formular.exceptions.ValidationError`
@@ -277,17 +326,24 @@ class Field(object):
             raise RuntimeError("Fields must be bound to be validated.")
         self.reset()
         errors = []
-        try:
-            self.value = self.convert(value)
-        except ValueError:
-            errors.append(self.error_messages["invalid"])
-            self.errors = ValidationError(*errors)
-            return self.is_valid
+        if convert:
+            try:
+                self.value = self.convert(value)
+            except ValueError:
+                errors.append(self.error_messages["invalid"])
+                self.errors = ValidationError(*errors)
+                return self.is_valid
+        else:
+            self.value = value
         for name, validator in self.validators.iteritems():
-            if not validator(self.form, self.value):
-                errors.append(self._format_error_message(
-                    self.error_messages[name], validator.last_args)
-                )
+            try:
+                if validator(self.form, self.value) == False:
+                    errors.append(self._format_error_message(
+                        self.error_messages[name],
+                        validator.last_args
+                    ))
+            except ValidationError as err:
+                errors.extend(err.args)
         self.errors = ValidationError(*errors)
         return self.is_valid
 
@@ -393,12 +449,13 @@ class Mapping(Field):
             rv[name] = field.convert(
                 value.get(name, field.to_primitive(field.default))
             )
+        rv.update(value)
         return rv
 
     def to_primitive(self, value):
         rv = OrderedDict()
         for name, field in self.mapped_fields.iteritems():
-            rv[name] = field.primitive_value
+            rv[name] = field.to_primitive(value[name])
         return rv
 
     def validate(self, value):
@@ -407,17 +464,21 @@ class Mapping(Field):
         if hasattr(self, "form"):
             self.bind(self.form)
         errors = OrderedDict()
-        self.value = OrderedDict()
+        converted_value = OrderedDict()
         for field_name, field in self.mapped_fields.iteritems():
-            primitive_value = value.get(
-                field_name,
-                field.to_primitive(field.default)
-            )
-            if not field.validate(primitive_value):
+            primitive = value.get(field_name)
+            if primitive is None:
+                primitive = field.to_primitive(field.default)
+            if not field.validate(primitive):
                 errors[field_name] = field.errors
-            self.value[field_name] = field.value
-        self.errors = MultipleValidationErrors(errors)
-        return self.is_valid
+            converted_value[field_name] = field.value
+        converted_value.update(value)
+        errors = MultipleValidationErrors(errors)
+        if errors:
+            self.value = converted_value
+            self.errors = errors
+            return self.is_valid
+        return Field.validate(self, converted_value, convert=False)
 
     def __repr__(self):
         return "{0}({1!r}, {2})".format(
@@ -429,8 +490,8 @@ class SizedField(Field):
     Represents a field, which value has a length.
 
     Available validators for this field are:
-        - :func:`formular.validators.min_length`
-        - :func:`formular.validators.max_length`
+        - :func:`~formular.validators.min_length`
+        - :func:`~formular.validators.max_length`
     """
     validator_factories = {
         "min_length": validators.min_length,
@@ -445,11 +506,38 @@ class SizedField(Field):
 class TextField(SizedField):
     """
     Represents a field which holds text.
+
+    :param default:
+        The default value for this field. If the given value contains a newline
+        a :class:`~formular.widgets.Textarea` is used as a widget.
+
+    :param multiline:
+        Forces the widget to be a :class:`~formular.widgets.Textarea`.
+
+        .. versionadded:: 0.2
+
+    .. versionchanged:: 0.2
+       Automatic change to :class:`~formular.widgets.Textarea` if the default
+       value contains a newline.
     """
-    error_messages = {
-        "min_length": _(u"Must be at least {initial_value} characters long."),
-        "max_length": _(u"Must be shorter then {initial_value} characters.")
+    validator_factories = {
+        "is_email": validators.is_email
     }
+    error_messages = {
+        "min_length" : _(u"Must be at least {initial_value} characters long."),
+        "max_length" : _(u"Must be shorter then {initial_value} characters."),
+        "is_email"   : _(u"An e-mail address is required.")
+    }
+
+    def __init__(self, multiline=False, default=u"", **kwargs):
+        SizedField.__init__(self, default=default, **kwargs)
+        self.multiline = multiline
+
+    @property
+    def widget(self):
+        if "\n" in self.default or self.multiline:
+            return Textarea
+        return TextInput
 
 class PasswordField(TextField):
     """
@@ -462,8 +550,8 @@ class ComparableField(Field):
     Represents a field with a comparable value.
 
     Available validators for this field are:
-        - :func:`formular.validators.min_value`
-        - :func:`formular.validators.max_value`
+        - :func:`~formular.validators.min_value`
+        - :func:`~formular.validators.max_value`
     """
     validator_factories = {
         "min_value": validators.min_value,
@@ -542,7 +630,28 @@ class BooleanField(Field):
             return True
         elif value == u"False":
             return False
-        raise ValueError()
+        raise ValueError("expected True or False, got: {0!r}".format(value))
+
+class SubmitField(Field):
+    """
+    Represents a submit button.
+
+    .. versionadded:: 0.2
+    """
+    default = False
+
+    widget = SubmitInput
+
+    def __init__(self, label, **kwargs):
+        Field.__init__(self, label=label, **kwargs)
+
+    def convert(self, value):
+        if value == self.label:
+            return True
+        raise ValueError("expected {0!r}, got: {1!r}".format(self.label, value))
+
+    def to_primitive(self, value):
+        return self.label
 
 class ChoiceField(Field):
     """
@@ -571,7 +680,7 @@ class ChoiceField(Field):
         for choice, _ in _iter_choices(self.choices):
             if _value_matches_choice(value, choice):
                 return choice
-        raise ValueError()
+        raise ValueError("{0!r} is not among the possible choices".format(value))
 
     def __repr__(self):
         return "{0}({1}, {2})".format(
@@ -727,6 +836,96 @@ class Seperated(Multiple):
             self._base_repr()
         )
 
+class ReCAPTCHAField(Field):
+    """
+    Represents a reCAPTCHA.
+
+    :param verify_server:
+        The URI to which the verification request is sent.
+
+    :param api_server:
+        The server which provides the api for the captcha field.
+
+    .. warning:: Only one ReCAPTCHAField can be bound to a form.
+
+    .. versionadded:: 0.2
+    """
+    widget = ReCAPTCHAWidget
+
+    def __init__(self,
+            label=u"Captcha",
+            verify_server="http://api-verify.recaptcha.net/verify",
+            api_server="http://api.recaptcha.net/",
+            **kwargs):
+        self.validator_factories = {}
+        self.error_messages = {
+            "invalid": _(u"The captcha is invalid.")
+        }
+        Field.__init__(self, label=label, **kwargs)
+        self.verify_server = verify_server
+        self.api_server = api_server
+
+    @property
+    def value(self):
+        """
+        ``True`` as long as the field is valid.
+        """
+        return self.is_valid
+
+    def bind(self, form, name=None):
+        for field in form.fields.itervalues():
+            if isinstance(field, self.__class__) and not field is self:
+                raise RuntimeError(
+                    "Only one ReCAPTCHAField can be bound to a form"
+                )
+        Field.bind(self, form, name=name)
+
+    def to_primitive(self, value):
+        if value is self.default:
+            return ""
+        raise NotImplementedError()
+
+    def convert(self, value):
+        """
+        This method is not implemented.
+        """
+        raise NotImplementedError(
+            "{0}.convert()".format(self.__class__.__name__)
+        )
+
+    def _set_invalid(self):
+        self.errors = ValidationError(self._format_error_message(
+            self.error_messages["invalid"],
+            {}
+        ))
+
+    def validate(self, value=None):
+        if not self.is_bound:
+            raise RuntimeError("Fields must be bound to be validated.")
+        self.reset()
+        try:
+            challenge = self.form.data["recaptcha_challenge_field"]
+            response = self.form.data["recaptcha_response_field"]
+        except KeyError:
+            self._set_invalid()
+            return self.value
+        request = urllib2.Request(
+            self.verify_server,
+            data=urlencode({
+                "privatekey" : self.form.recaptcha_private_key.encode("utf-8"),
+                "remoteip"   : self.form.remote_addr.encode("utf-8"),
+                "challenge"  : challenge.encode("utf-8"),
+                "reponse"    : challenge.encode("utf-8")
+            })
+        )
+        with closing(urllib2.urlopen(request)) as response:
+            rv = response.read().splitlines()
+        if rv and rv[0] == "true":
+            return self.value
+        self._set_invalid()
+        return self.value
+
+
 if HAS_BABEL:
     class TimeField(ComparableField):
         """
@@ -757,7 +956,7 @@ if HAS_BABEL:
                     locale=self.form.translations.locale
                 )
             except IndexError:
-                raise ValueError()
+                raise ValueError("{0!r} couldn't be parsed".format(value))
             # FIXME: This should be done upstream in babel.
             if "pm" in value.lower() and result.hour != 12:
                 result = result.replace(hour=result.hour + 12)
@@ -789,13 +988,13 @@ if HAS_BABEL:
             try:
                 return parse_date(value, locale=self.form.translations.locale)
             except IndexError:
-                raise ValueError()
+                raise ValueError("{0!r} couldn't be parsed".format(value))
 
 __all__ = [
     "Field", "Mapping", "SizedField", "TextField", "PasswordField",
     "ComparableField", "NumberField", "IntegerField", "FloatField",
-    "ComplexField", "DecimalField", "BooleanField", "ChoiceField",
-    "MultiChoiceField", "Multiple", "Seperated"
+    "DecimalField", "BooleanField", "ChoiceField", "MultiChoiceField",
+    "Multiple", "Seperated", "ReCAPTCHAField", "SubmitField"
 ]
 
 if HAS_BABEL:
