@@ -9,6 +9,9 @@
 """
 from cgi import escape
 from itertools import imap, chain
+import json
+from urllib import urlencode
+from textwrap import dedent
 
 from formular._utils import _is_choice_selected, _iter_choices, _make_name
 
@@ -112,10 +115,33 @@ class Label(Renderable):
         self.text = text
         self.field_id = field_id
 
-    def render(self):
+    def as_th(self, **attrs):
+        """
+        Returns the :attr:`text` in a `th`.
+
+        .. versionadded:: 0.2
+        """
+        return self.builder.th(self.text, **attrs)
+
+    def as_dt(self, **attrs):
+        """
+        Returns the rendered label in a `dt`.
+
+        .. versionadded:: 0.2
+        """
+        return self.builder.dt(self.render(), **attrs)
+
+    def render(self, **attrs):
+        """
+        Returns a `label` element.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `label`.
+        """
+        attrs["for"] = self.field_id
         return self.builder.label(
             self.text,
-            **{"for": self.field_id}
+            **attrs
         )
 
     def __repr__(self):
@@ -156,11 +182,44 @@ class Widget(Renderable):
     def id(self):
         return u"f_{0}".format(self.name.replace(u".", u"_"))
 
+    def as_dd(self):
+        """
+        Returns the label as `dt` and the rendered widget as `dd`.
+
+        .. versionadded:: 0.2
+        """
+        return self.label.as_dt() + self.builder.dd(self.render())
+
+    def as_td(self):
+        """
+        Returns the rendered widget in a `td`.
+
+        .. versionadded:: 0.2
+        """
+        return self.builder.td(self.render())
+
     def __repr__(self):
         return "{0}({1!r}, {2!r}, dialect={3!r})".format(
             self.__class__.__name__, self.field, self.name, self.dialect
         )
 
+class InlineWidgetMixin(object):
+    """
+    This mixin adds generic methods for widgets which get rendered to inline
+    elements.
+
+    .. versionadded:: 0.2
+    """
+    # each docstring should contain a versionadded/-changed directive as the
+    # mixin itself is not documented
+    def as_li(self):
+        """
+        Returns the rendered widget in a `li`.
+
+        .. versionadded:: 0.2
+        """
+        return self.builder.li(self.render())
+
 class ChildWidget(Widget):
     """
     Represents a value of field as a UI-Element.
@@ -175,17 +234,24 @@ class ChildWidget(Widget):
     def __repr__(self):
         return "{0}({1!r})".format(self.__class__.__name__, self.parent)
 
-class Input(Widget):
+class Input(Widget, InlineWidgetMixin):
     """
     The base class for input widgets.
     """
     #: Specifies the type of the input widget.
     type = None
 
-    def render(self):
-        attrs = {}
+    def render(self, **attrs):
+        """
+        Returns the rendered `input` element.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `input`.
+        """
         if self.type == "checkbox" and self.field.value:
             attrs["checked"] = "checked"
+        elif self.type == "submit":
+            attrs["value"] = self.label.text
         else:
             attrs["value"] = self.value if self.type != "password" else None
         return self.builder.input(
@@ -209,6 +275,15 @@ class PasswordInput(Input):
     """
     type = "password"
 
+class SubmitInput(Input):
+    """
+    Represents a :class:`formular.fields.SubmitField` instance as an
+    UI-Element.
+
+    .. versionadded:: 0.2
+    """
+    type = "submit"
+
 class Checkbox(Input):
     """
     Represents a :class:`formular.fields.BooleanField` instance as an
@@ -229,9 +304,15 @@ class Textarea(Widget):
     lines of text in the UI.
     """
     def render(self, **attrs):
+        """
+        Returns the rendered `textarea` element.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `textarea`.
+        """
         return self.builder.textarea(self.value, name=self.name, **attrs)
 
-class InputGroupMember(ChildWidget):
+class InputGroupMember(ChildWidget, InlineWidgetMixin):
     """
     Represents a value of :class:`formular.fields.ChoiceField` instance.
     """
@@ -258,6 +339,9 @@ class InputGroupMember(ChildWidget):
         return _is_choice_selected(self.parent, self.value)
 
     def render(self, **attrs):
+        """
+        Returns the rendered `input` element.
+        """
         return self.builder.input(
             id=self.id,
             type=self.type,
@@ -289,15 +373,21 @@ class GroupCheckbox(InputGroupMember):
 class ListWidgetMixin(Renderable):
     def as_ul(self, **attrs):
         """
-        Renders the widget as an unordered list.
+        Renders the widget as an `ul`.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `ul`.
         """
-        return self._as_list(self.builder.ul)
+        return self._as_list(self.builder.ul, **attrs)
 
     def as_ol(self, **attrs):
         """
-        Renders the widget as an ordered list.
+        Renders the widget as an `ol`.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `ul`.
         """
-        return self._as_list(self.builder.ol)
+        return self._as_list(self.builder.ol, **attrs)
 
     def _as_list(self, list_factory, **attrs):
         raise NotImplementedError(self.__class__.__name__ + "_as_list")
@@ -316,11 +406,17 @@ class InputGroup(Widget, ListWidgetMixin):
             self.choices.append(widget)
             self.subwidgets[value] = widget
 
-    def _as_list(self, list_factory):
-        return list_factory(*imap(self.builder.li, self.choices))
+    def _as_list(self, list_factory, **attrs):
+        return list_factory(*(choice.as_li() for choice in self.choices), **attrs)
 
-    def render(self):
-        return self.as_ul()
+    def render(self, **attrs):
+        """
+        Returns the rendered `ul` element.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `ul`.
+        """
+        return self.as_ul(**attrs)
 
 class RadioButtonGroup(InputGroup):
     """
@@ -366,27 +462,32 @@ class MappingWidget(Widget):
         for name in self.field.mapped_fields:
             yield self[name]
 
-    def as_dl(self):
+    def as_dl(self, **attrs):
         """
-        Renders the subwidgets as a definition list, where each term is the
-        rendered field label and the corresponding description the rendered
-        field.
-        """
-        return self.builder.dl(
-            *(self.builder.dt(field.label.render()) + self.builder.dd(field)
-              for field in self)
-        )
+        Renders the subwidgets as a `dl`.
 
-    def as_tr(self):
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `dl`
         """
-        Renders the subwidgets as a table row.
-        """
-        return self.builder.tr(
-            *(self.builder.td(field.render()) for field in self)
-        )
+        return self.builder.dl(*(field.as_dd() for field in self), **attrs)
 
-    def render(self):
-        return self.as_dl()
+    def as_tr(self, **attrs):
+        """
+        Renders the subwidgets as a `tr`
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `tr`.
+        """
+        return self.builder.tr(*(field.as_td() for field in self), **attrs)
+
+    def render(self, **attrs):
+        """
+        Returns the rendered `dl` element.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `dl`.
+        """
+        return self.as_dl(**attrs)
 
 class MultipleWidget(Widget, ListWidgetMixin):
     """
@@ -396,22 +497,49 @@ class MultipleWidget(Widget, ListWidgetMixin):
         Widget.__init__(self, *args, **kwargs)
         self.subfield = self.field.field
 
-    @property
-    def subwidget(self):
-        return self.subfield.as_widget()
+    def iter_subwidgets(self):
+        """
+        Yields a subwidget for every value.
 
-    def _as_list(self, list_factory):
-        rendered = []
+        .. versionadded:: 0.2
+        """
         for i, value in enumerate(self.value):
             self.subfield.validate(value)
             self.subfield.name = _make_name(self.field.name, unicode(i))
-            rendered.append(self.subwidget.render())
-        self.subfield.reset()
+            yield self.subfield.as_widget()
+            self.subfield.reset()
         self.subfield.name = None
-        return list_factory(*rendered)
 
-    def render(self):
-        return self.as_ul()
+    def _as_list(self, list_factory, **attrs):
+        return list_factory(
+            *(subwidget.as_li() for subwidget in self.iter_subwidgets()),
+            **attrs
+        )
+
+    def as_table(self, **attrs):
+        """
+        Renders the widget as a `table` if possible otherwise a
+        :exc:`TypeError` is raised.
+
+        .. versionadded:: 0.2
+        """
+        if not hasattr(self.subfield.as_widget(), "as_tr"):
+            raise TypeError(
+                "Can't create a table with subwidgets that can't be rendered "
+                "to rows."
+            )
+        head = [w.label.as_th() for w in self.subfield.as_widget()]
+        body = [w.as_tr() for w in self.iter_subwidgets()]
+        return self.builder.table(*(head + body), **attrs)
+
+    def render(self, **attrs):
+        """
+        Returns the rendered `ul` element.
+
+        .. versionadded:: 0.2
+           Now supports arbitary attributes for the `ul`
+        """
+        return self.as_ul(**attrs)
 
 class FormWidget(Renderable):
     """
@@ -438,6 +566,9 @@ class FormWidget(Renderable):
         .. note:: This method can be used as a
                   `jinja macro <http://jinja.pocoo.org/2/documentation/templates#call>`_
                   The caller will have to take care of the visible fields.
+
+        .. versionadded:: 0.2
+           Accepts now arbitary attributes for the `form`
         """
         hidden = u"".join(
             self.builder.input(
@@ -447,16 +578,30 @@ class FormWidget(Renderable):
             ) for name, value in self.form.hidden_data.iteritems()
         )
         caller = attrs.pop("caller", None)
-        if attrs:
-            raise TypeError("Unexpected keyword argument.")
         if caller:
             body = caller()
         else:
-            body = self.form._root_field.as_widget().as_dl()
+            root_widget = self.form._root_field.as_widget()
+            body = root_widget.as_dl()
+            if not any(isinstance(w, SubmitInput) for w in root_widget):
+                _ = self.form.translations.gettext
+                body += self.builder.div(
+                    self.builder.input(
+                        type="submit",
+                        name="_formular_submit",
+                        value=_(u"Submit")
+                    ),
+                    **{"class": "actions"}
+                )
         if hidden:
             # <input> fields must not be children of a <form> tag
             body += u'<div style="display: none">{0}</div>'.format(hidden)
-        return self.builder.form(body, action=self.form.action, method=method)
+        return self.builder.form(
+            body,
+            action=self.form.action,
+            method=method,
+            **attrs
+        )
 
 class ValidationErrorWidget(ListWidgetMixin):
     """
@@ -480,13 +625,15 @@ class ValidationErrorWidget(ListWidgetMixin):
                 (self.form.fields[key].label, errors)
                 for key, errors in self._errors.iteritems()
             )
+        else:
+            self._errors = self._errors.values()
         return self._errors
 
     def _as_list(self, list_factory, **attrs):
         if hasattr(self.errors, "itervalues"):
             errors = chain(*self.errors.itervalues())
         else:
-            errors = self.errors
+            errors = chain(*self.errors)
         return list_factory(
             *imap(self.builder.li, errors),
             **attrs
@@ -494,7 +641,7 @@ class ValidationErrorWidget(ListWidgetMixin):
 
     def as_dl(self, **attrs):
         """
-        Renders the validation error as a definition list.
+        Renders the validation error as a `dl`.
 
         This method may raise a :exc:`RuntimeError` if the form has top-level
         errors.
@@ -510,9 +657,8 @@ class ValidationErrorWidget(ListWidgetMixin):
 
     def render(self, **attrs):
         """
-        Renders the validation error as a div with a headline. The errors
-        are displayed as an unordered list if they cannot be displayed as a
-        definition list.
+        Renders the validation error as a `div` with a `h1`. The errors
+        are displayed as an `ul` if they cannot be displayed as a `dl`.
 
         If the form has no validation errors this method returns an empty
         string.
@@ -536,8 +682,56 @@ class ValidationErrorWidget(ListWidgetMixin):
             self.dialect
         )
 
+class ReCAPTCHAWidget(Widget):
+    """
+    Represents a :class:`formular.fields.ReCAPTCHAField` as an UI-Element.
+
+    .. versionadded:: 0.2
+    """
+
+    def render(self):
+        """
+        Renders the field using the reCAPTCHA javascript api or an `iframe` if
+        the browser doesn't support javascript.
+        """
+        _ = self.field.form.translations.gettext
+        options = urlencode({
+            "k": self.field.form.recaptcha_public_key
+        })
+        return dedent(u"""\
+        <script type="text/javascript">var RecaptchaOptions = {options};</script>
+        <script type="text/javascript" src={script_url}></script>
+        <noscript>
+          <div>
+              <iframe src="{frame_url}" height="300" width="500" frameborder="0">
+              </iframe>
+          </div>
+          <div>
+              <textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
+              <input type="hidden" name="recaptcha_response_field" value="manual_challenge">
+          </div>
+        </noscript>
+        """.format(
+            script_url="{0}challenge?{1}".format(self.field.api_server, options),
+            frame_url="{0}noscript?{1}".format(self.field.api_server, options),
+            options=json.dumps({
+                "theme": "clean",
+                "custom_translation": {
+                    "visual_challenge"    : _(u"Get a visual challenge"),
+                    "audio_challenge"     : _(u"Get an audio challenge"),
+                    "refresh_btn"         : _(u"Get a new challenge"),
+                    "instructions_visual" : _(u"Type the two words: "),
+                    "instructions_audio"  : _(u"Type what you hear: "),
+                    "help_btn"            : _(u"Help"),
+                    "play_again"          : _(u"Play sound again"),
+                    "cant_hear_this"      : _(u"Download sound as MP3"),
+                    "incorrect_try_again" : _(u"Incorrect, try again")
+                }
+            })
+        )).replace("\n", "")
+
 __all__ = [
     "TextInput", "PasswordInput", "Checkbox", "Textarea", "RadioButtonGroup",
     "CheckboxGroup", "MappingWidget", "MultipleWidget", "FormWidget",
-    "ValidationErrorWidget"
+    "ValidationErrorWidget", "ReCAPTCHAWidget", "SubmitInput"
 ]
